package models

import "gogs.mikescher.com/BlackForestBytes/goext/rfctime"

type OrgType string //@enum:type

const (
	OrgTypeCommunity   OrgType = "COMMUNITY"   // Gemeinde
	OrgTypeAssociation OrgType = "ASSOCIATION" // Verein
	OrgTypeBusiness    OrgType = "BUSINESS"    // Unternehmen
	OrgTypeArtist      OrgType = "ARTIST"      // Künstler
	OrgTypePress       OrgType = "PRESS"       // Presse
)

type Organization struct {
	OrganizationID      OrganizationID           `json:"id"                  bson:"_id,omitempty"`
	Name                string                   `json:"name"                bson:"name"`
	FrontendID          *string                  `json:"frontendID"          bson:"frontendID"`
	FrontendEnabled     bool                     `json:"frontendEnabled"     bson:"frontendEnabled"`
	CreationTime        rfctime.RFC3339NanoTime  `json:"creationTime"        bson:"creationTime"`
	Type                OrgType                  `json:"type"                bson:"type"`
	Purpose             string                   `json:"purpose"             bson:"purpose"`
	Address             OrgUserAddress           `json:"address"             bson:"address"`
	CustomerNumber      string                   `json:"customerNumber"      bson:"customerNumber"`
	Banking             OrgUserBankingInfo       `json:"banking"             bson:"banking"`
	Phone               string                   `json:"phone"               bson:"phone"`
	Comment             string                   `json:"comment"             bson:"comment"`
	AvatarImageID       *BlobID                  `json:"avatarImageID"       bson:"avatarImageID"`
	BackgroundImageID   *BlobID                  `json:"backgroundImageID"   bson:"backgroundImageID"`
	ProfileData         OrgProfileData           `json:"profileData"         bson:"profileData"`
	LinkedOrganizations []OrganizationID         `json:"linkedOrganizations" bson:"linkedOrganizations"`
	Deleted             *rfctime.RFC3339NanoTime `json:"deleted"             bson:"deleted"`
}

func (t OrgType) ToFAPI() string {
	if t == OrgTypeCommunity {
		return "municipality"
	} else if t == OrgTypeAssociation {
		return "socialclub"
	} else if t == OrgTypeBusiness {
		return "corporation"
	} else if t == OrgTypeArtist {
		return "artist"
	} else if t == OrgTypePress {
		return "press"
	}

	return ""
}
